@@ -43,6 +43,8 @@ Why pydantic over standard dataclasses?
 
   - Built in parsing of json or yaml into nested models (ie when attributes are reference types)
 
+  TODO create linkml root class similar to from linkml_runtime.utils.yamlutils import YAMLRoot
+
 """
 
 from pathlib import Path
@@ -51,14 +53,14 @@ from typing import List, Optional, TextIO, Tuple, Union
 import typer
 from linkml.generators import PYTHON_GEN_VERSION
 from linkml.generators.pythongen import PythonGenerator
-from linkml.utils.formatutils import be, camelcase, split_line, wrapped_annotation
-from linkml_model.meta import (
+from linkml_runtime.linkml_model.meta import (
     ClassDefinition,
     ClassDefinitionName,
     EnumDefinition,
     SchemaDefinition,
     SlotDefinition,
 )
+from linkml_runtime.utils.formatutils import be, camelcase, split_line, wrapped_annotation
 
 
 class PydanticGen(PythonGenerator):
@@ -80,7 +82,7 @@ class PydanticGen(PythonGenerator):
             **kwargs,
         )
 
-    # Overriden methods
+    # Overridden methods
 
     def _sort_classes(self, clist: List[ClassDefinition]) -> List[ClassDefinition]:
         """
@@ -140,14 +142,16 @@ class PydanticGen(PythonGenerator):
 # description: {split_descripton}
 # license: {be(self.schema.license)}
 
+from __future__ import annotations
+
 import datetime
 import inspect
 import logging
-from collections import namedtuple
-from enum import Enum, unique
-from dataclasses import field
-from typing import Any, ClassVar, List, Optional, Union, Dict
 import re
+from collections import namedtuple
+from dataclasses import field
+from enum import Enum, unique
+from typing import Any, ClassVar, List, Optional, Union
 
 from pydantic import constr, validator
 from pydantic.dataclasses import dataclass
@@ -155,8 +159,7 @@ from pydantic.dataclasses import dataclass
 LOG = logging.getLogger(__name__)
 
 metamodel_version = "{self.schema.metamodel_version}"
-
-curie_regexp = r'^[a-zA-Z_]?[a-zA-Z_0-9-]*:([A-Za-z0-9_][A-Za-z0-9_.-]*[A-Za-z0-9_]*)?$'
+curie_regexp = r'^[a-zA-Z_]?[a-zA-Z_0-9-]*:([A-Za-z0-9_][A-Za-z0-9_.-]*[A-Za-z0-9./_]*)?$'
 curie_pattern = re.compile(curie_regexp)
 
 # Type Aliases
@@ -173,6 +176,7 @@ FrequencyValue = str
 PercentageFrequencyValue = float
 BiologicalSequence = str
 Quotient = float
+Bool = bool
 
 # Namespaces
 {self.gen_namespaces()}
@@ -188,8 +192,20 @@ Quotient = float
 # Classes
 {self.gen_classdefs()}
 
+# Update forward refs
+# see https://pydantic-docs.helpmanual.io/usage/postponed_annotations/
+{self.gen_forward_refs()}
+
 '''
 
+    def gen_forward_refs(self):
+        return '\n'.join(
+            [
+                f'{self.class_or_type_name(cls.name)}.__pydantic_model__.update_forward_refs()'
+                for cls in self._sort_classes(self.schema.classes.values())
+            ]
+        )
+
     def gen_classdef(self, cls: ClassDefinition) -> str:
         """ Generate python definition for class cls """
 
@@ -202,15 +218,29 @@ Quotient = float
             parent_class_and_mixins = ', '.join(
                 [self.formatted_element_name(parent, True) for parent in parents]
             )
+            # See https://github.com/linkml/linkml/issues/290
+            # We need a more intelligent way to order inherited classes
+            # default: parent, mixin_1, mixin_2, etc
+
             parent_class_and_mixins = f'({parent_class_and_mixins})'
         elif cls.mixins:
             # Seems fine but more curious if this ever happens
+            # Follow up - it does happen
             self.logger.warning(f"class {cls.name} has mixins {cls.mixins} but no parent")
             mixins = ', '.join([self.formatted_element_name(mixin, True) for mixin in cls.mixins])
             parent_class_and_mixins = f'({mixins})'
 
+        # This is a horrible hack to resolve our MRO issues in time for the koza hackathon
+        # see github issue above
+        if (
+            self.class_or_type_name(cls.name) == 'Behavior'
+            and parent_class_and_mixins == '(BiologicalProcess, OntologyClass, ActivityAndBehavior)'
+        ):
+            parent_class_and_mixins = '(BiologicalProcess, ActivityAndBehavior, OntologyClass)'
+
         slotdefs = self.gen_class_variables(cls)
 
+        # TODO replace this with root class
         entity_post_init = (
             f'\n\t{self._get_entity_post_init()}'
             if self.class_or_type_name(cls.name) == 'Entity'
@@ -375,7 +405,9 @@ Quotient = float
         We have a union of str, Curie, and the class, eg
         Entity -> str, Curie, Entity
 
-        TODO clean this up
+        TODO
+        Replace this with something more generalizable across
+        all LinkMl schemas
 
         :param rangelist: List of types from distal to proximal
         :return:
@@ -387,12 +419,12 @@ Quotient = float
             'category',
             'provided_by',
             'xref',
-            'has_qualitative_value',
-            'subclass_of',
-            'has_input',
-            'has_output',
-            'has_constituent',
-            'enabled_by',
+            #'has_qualitative_value',
+            #'subclass_of',
+            #'has_input',
+            #'has_output',
+            #'has_constituent',
+            #'enabled_by',
         ]:
             # id is here to override {ClassName}Id
             # The rest are due to import order errors in self._sort_classes
@@ -403,8 +435,6 @@ Quotient = float
 
         elif 'Entity' in rangelist:
             class_ref = f"Union[URIorCURIE, {rangelist[-1]}]" if len(rangelist) > 1 else base
-        elif 'Bool' == rangelist[-1]:
-            class_ref = 'bool'
         else:
             if len(rangelist) > 1 and rangelist[-1] == 'IriType':
                 class_ref = 'IriType'
@@ -576,13 +606,13 @@ Predicate = namedtuple(
                     'id',
                     'category',
                     'provided_by',
-                    'xref',
-                    'has_qualitative_value',
-                    'subclass_of',
-                    'has_input',
-                    'has_output',
-                    'has_constituent',
-                    'enabled_by',
+                    #'xref',
+                    #'has_qualitative_value',
+                    #'subclass_of',
+                    #'has_input',
+                    #'has_output',
+                    #'has_constituent',
+                    #'enabled_by',
                 ]
                 or ('URIorCURIE' in rangelist and rangelist[-1] != 'IriType')
             ) and slotname != 'predicate':
@@ -638,7 +668,7 @@ Predicate = namedtuple(
         :return:
         """
         return f'''
-    @validator('{slotname}')
+    @validator('{slotname}', allow_reuse=True)
     def check_{slotname}_prefix(cls, value):
         check_curie_prefix({cls_name}, value)
         return value'''
@@ -650,7 +680,7 @@ Predicate = namedtuple(
         :return:
         """
         return f'''
-    @validator('{slotname}')
+    @validator('{slotname}', allow_reuse=True)
     def convert_{slotname}_to_list_check_curies(cls, value):
         return convert_scalar_to_list_check_curies({cls_name}, value)'''
 
@@ -663,7 +693,7 @@ Predicate = namedtuple(
         :return:
         """
         validator = f'''
-    @validator('{slotname}')
+    @validator('{slotname}', allow_reuse=True)
     def validate_required_{slotname}(cls, value):
         check_value_is_not_none("{slotname}", value)
 '''
